@@ -13,17 +13,17 @@ log = logging.getLogger(__name__)
 
 class RetryingExecutor(TaskExecutor):
     def __init__(self,
-                 executor,
+                 downstream_executor,
                  retry_pred=lambda e: not e.success,
                  retries=3):
-        self.executor = executor
+        self.executor = downstream_executor
         self.retries = retries
         self.retry_pred = retry_pred
 
         self.task_retries = m()
         self.task_retries_lock = Lock()
 
-        self.src_queue = executor.get_event_queue()
+        self.src_queue = downstream_executor.get_event_queue()
         self.dest_queue = Queue()
         self.stopping = False
 
@@ -41,25 +41,22 @@ class RetryingExecutor(TaskExecutor):
         )
 
     def retry(self, event):
-        current_retry_attempt = self.task_retries[event.task_id]
-        # This task has been killed manually
-        if current_retry_attempt == -1:
-            return False
-
-        if current_retry_attempt == self.retries:
+        retries_remaining = self.task_retries[event.task_id]
+        if retries_remaining <= 0:
             return False
 
+        total_retries = self._task_or_executor_retries(event.task_config)
         log.info(
             'Retrying task {}, {} of {}, fail event: {}'.format(
-                event.task_config.name, current_retry_attempt,
-                self.retries, event.raw
+                event.task_config.name, total_retries - retries_remaining + 1,
+                total_retries, event.raw
             )
         )
 
         with self.task_retries_lock:
             self.task_retries = self.task_retries.set(
                 event.task_id,
-                current_retry_attempt + 1
+                retries_remaining - 1
             )
         self.run(event.task_config)
 
@@ -69,6 +66,11 @@ class RetryingExecutor(TaskExecutor):
         while True:
             while not self.src_queue.empty():
                 e = self.src_queue.get()
+
+                if e.kind != 'task':
+                    self.dest_queue.put(e)
+                    continue
+
                 # This is to remove trailing '-retry*'
                 original_task_id = '-'.join([item for item in
                                              e.task_id.split('-')[:-1]])
@@ -81,10 +83,6 @@ class RetryingExecutor(TaskExecutor):
                 # Set the task id back to original task_id
                 e = self._restore_task_id(e, original_task_id)
 
-                if e.kind != 'task':
-                    self.dest_queue.put(e)
-                    continue
-
                 e = self.event_with_retries(e)
 
                 if e.terminal:
@@ -107,7 +105,9 @@ class RetryingExecutor(TaskExecutor):
         if task_config.task_id not in self.task_retries:
             with self.task_retries_lock:
                 self.task_retries = self.task_retries.set(
-                    task_config.task_id, 1)
+                    task_config.task_id,
+                    self._task_or_executor_retries(task_config)
+                )
         self.executor.run(self._task_config_with_retry(task_config))
 
     def kill(self, task_id):
@@ -154,3 +154,7 @@ class RetryingExecutor(TaskExecutor):
         if attempt == self.task_retries[original_task_id]:
             return True
         return False
+
+    def _task_or_executor_retries(self, task_config):
+        return task_config.retries \
+            if 'retries' in task_config else self.retries
